#!/usr/bin/env python

###############################################################################
# aiwendil -- see http://www.esiea-recherche.eu/~desnos                       #
#                                                                             #
# The project aiwendil is (C) 2009 : desnos at esiea.fr                       #
# This program is free software;                                              #
# you can redistribute it and/or modify it under the terms of the GNU         #
# General Public License as published by the Free Software Foundation;        #
# Version 2. This guarantees your right to use, modify, and                   #
# redistribute this software under certain conditions.                        #
#                                                                             #
# Source is provided to this software because we believe users have a         #
# right to know exactly what a program is going to do before they run         #
# it.                                                                         #
#                                                                             #
# This program is distributed in the hope that it will be                     #
# useful, but WITHOUT ANY WARRANTY; without even the implied                  #
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                     #
# PURPOSE. See the GNU General Public License for more details (              #
# http://www.gnu.org/copyleft/gpl.html ).                                     #
#                                                                             #
# Dependency :                                                                #
#       - ptrace        http://fusil.hachoir.org/trac/wiki/Ptrace             #
#                                                                             #
###############################################################################

import getopt, os, cmd, sys, xml.parsers.expat

from boxrk import BoxRootkit

from logging import (log, NOTSET, CRITICAL, DEBUG, INFO, WARNING, ERROR)

from log import NOLOG, LOGFILE, MyLog

class newFile :
    def __init__(self, name) :
        self.name = name

        self.base = "/"
        self.baseinode = 0
        self.type = 8
        self.dev = 0
        self.rdev = 0
        
        self.seek = 0
        self.inode = 0
        self.mode = 0
        self.nlink = 0
        
        self.uid = 0
        self.gid = 0

        self.size = 0

        self.atime = 0
        self.atimensec = 0
        self.mtime = 0
        self.mtimensec = 0
        self.ctime = 0
        self.ctimensec = 0

        self.blocks = 0
        self.blksize = 0
        
        self.getxattr = [ -61, -95 ]
        self.posgetxattr = 0

        self.content = ""

    def view(self) :
        log(LOGFILE, "Name %s Type %s Base %s Inode %d Mode %d : %s" % (self.name, self.type, self.base, self.inode, self.mode, self.content))
        
class XmlConfig :
    def __init__(self, conf, b) :
        self.conf = conf
        self.b = b
        self.current_element = None
        self.injectfile = False
        self.injectfiles = {}
        
    def start_element(self, name, attrs):
        #print 'Start element:', name, attrs
        self.current_element = name
        if name == "injectfile" :
            filename = attrs["name"].strip()
            self.injectfiles[filename] = newFile(filename)
            self.injectfile = True
            self.current_file = filename
            
    def end_element(self, name):
        #print 'End element:', name
        self.current_element = ""
        if name == "injectfile" :
            self.injectfile = False
            
    def char_data(self, data):
        #print 'Character data :', self.current_element, repr(data)
        if self.current_element == "hidefiles" :
            self.b.filter_files(data)
            
        if self.injectfile :
            if self.current_element == "base" :
                data = data.split()
                self.injectfiles[self.current_file].base = data[0]
                self.injectfiles[self.current_file].baseinode = long(data[1])
            elif self.current_element == "type" :
                self.injectfiles[self.current_file].type = int(data)
            elif self.current_element == "dev" :
                self.injectfiles[self.current_file].dev = int(data)
            elif self.current_element == "rdev" :
                self.injectfiles[self.current_file].rdev = int(data)
            elif self.current_element == "inode" :
                self.injectfiles[self.current_file].inode = long(data)
            elif self.current_element == "mode" :
                self.injectfiles[self.current_file].mode = int(data)
            elif self.current_element == "nlink" :
                self.injectfiles[self.current_file].nlink = int(data)
            elif self.current_element == "uid" :
                self.injectfiles[self.current_file].uid = int(data)
            elif self.current_element == "gid" :
                self.injectfiles[self.current_file].gid = int(data)
            elif self.current_element == "atime" :
                data = data.split()
                self.injectfiles[self.current_file].atime = long(data[0])
                self.injectfiles[self.current_file].atimensec = long(data[1])
            elif self.current_element == "mtime" :
                data = data.split()
                self.injectfiles[self.current_file].mtime = long(data[0])
                self.injectfiles[self.current_file].mtimensec = long(data[1])
            elif self.current_element == "ctime" :
                data = data.split()
                self.injectfiles[self.current_file].ctime = long(data[0])
                self.injectfiles[self.current_file].ctimensec = long(data[1])
            elif self.current_element == "blocks" :
                self.injectfiles[self.current_file].blocks = long(data)
            elif self.current_element == "blksize" :
                self.injectfiles[self.current_file].blksize = long(data)
            elif self.current_element == "content" :
                self.injectfiles[self.current_file].content = data + "\n"
                self.injectfiles[self.current_file].size = len(data)

    def parse(self) :           
        p = xml.parsers.expat.ParserCreate()

        p.StartElementHandler = self.start_element
        p.EndElementHandler = self.end_element
        p.CharacterDataHandler = self.char_data

        fd = open(self.conf, "r")
        l = fd.readlines()
        fd.close()

        bufxml = ""
        for i in l :
            bufxml += i

        p.Parse(bufxml, 1)

        for i in self.injectfiles :
            self.injectfiles[i].view()
            self.b.inject_file(self.injectfiles[i])
            
            
if __name__ == "__main__" :
    mylog = MyLog()
    mylog.setLevel(NOLOG)
    #mylog.setLevel(INFO)
    #mylog.setLevel(LOGFILE)
    
    b = BoxRootkit(False, False)
    xc = XmlConfig("./aiwendil.xml", b)
    xc.parse()
    
    b.add_program(sys.argv[1:])
    b.walk()
