from box import *

class BoxProt(Box) :
    def setup_pdf(self) :
        self.pdf = True
        
    def add_program(self, args, is_gui=False) :
        self.emul = Emulate(self.oemul, self.pdf)
        if self.filterfiles.only != () :
            self.emul.set_filterfiles(self.filterfiles)

        self.program = args
        self.is_gui = is_gui

    def filter_askme(self, args) :
        self.askme = True
        if args == "no" :
            self.askme_para = False
        else :
            self.askme_para = True
            
        info("%sFilter askme %s para %s%s" % (Color.purple, self.askme, self.askme_para, Color.normal))

    def filter_resetfiles(self) :
        self.filterfiles.reset()
        
    def filter_reset(self, args) :
        self.askme = False
        if not args :
            self.filter.reset()

        else :
            for i in args.split() :
                i = i.strip()
                self.filter.reset(i)

    def filter_files(self, args) :
        for i in args.split() :
            i = i.strip()
            self.filterfiles.add_expr(i)
            
    def filter_sockets(self) :
        self.raw_socketcall = False
        self.filter.add_sockets()

    def filter_syscalls(self, args) :
        for i in args.split() :
            i = i.strip()
            self.filter.add_expr("only", i)

    def filter_syscalls_exclude(self, args) :
        for i in args.split() :
            i = i.strip()
            self.filter.add_expr("exclude", i)
            
    def filter_badsyscalls(self) :
         self.filter.add_badsyscalls()
        
    def filter_delsyscalls(self, args) :
        for i in args.split() :
            i = i.strip()
            self.filter.del_expr("only", i)

    def filter_delsyscalls_exclude(self, args) :
        for i in args.split() :
            i = i.strip()
            self.filter.del_expr("exclude", i)
                
    def view_filter(self) :
        self.filter.view("only")
        self.filter.view("exclude")

    def view_filterfiles(self) :
        self.filterfiles.view()
        
    def add_display(self, d) :
        self.d = d

    def displaySyscall(self, process, syscall, is_exit=True):
        if self.filter.isPresent("exclude", syscall.name) :
            return

        if self.askme or self.filter.isPresent("only", syscall.name) :
            pass
        elif not self.filter.isExist("only") :
            pass
        else :
            return
         
        text = syscall.format()
        if is_exit:
            exit_text = syscall.result_text
            text = "%-40s = %s" % (text, exit_text)

        info("%s[%s]%s %s%s" % (Color.yellow, process.pid, Color.red, text, Color.normal))
 
    def syscallTrace(self, process):
        # First query to break at next syscall
        self.prepareProcess(process)

        while True:
            # No more process? Exit
            if not self.debugger:
                break

            # Wait until next syscall enter
            try:
                event = self.debugger.waitSyscall()
                process = event.process
            except ProcessExit, event:
                self.processExited(event)
                continue
            except ProcessSignal, event:
                event.display()
                process.syscall(event.signum)
                continue
            except NewProcessEvent, event:
                self.newProcess(event)
                continue
            except ProcessExecution, event:
                self.processExecution(event)
                continue

            # Process syscall enter or exit
            self.syscall(process)

    def syscall_askme(self, process, syscall) :
        if self.filter.isPresent("only", syscall.name) :
            self.displaySyscall(process, syscall, False)
            info("%sAuthorize %s ?%s" % (Color.red, syscall.name, Color.normal))
            l = sys.stdin.readline()
            l = l.strip()
            if (l == 'y') :
                info("%s%sAuthorized %s...%s" % (Color.yellow, Color.uline, syscall.name, Color.normal))
                return 0
            else :
                info("%s%sBlocked %s...%s" % (Color.yellow, Color.uline, syscall.name, Color.normal))
                return 1

        return 0
    
    def syscall(self, process):
        state = process.syscall_state
        syscall = state.event(self.syscall_options)

        if process in self.active_syscalls:
            # syscall exit

            # Emulating syscall
            val = self.emul.emulate_syscall_exit(process, syscall)

            del self.active_syscalls[process]
            debug("%s exits %s" % (process, syscall))

            # Display syscall
            if self.oemul :
                self.displaySyscall(process, syscall)
        else:
            # syscall enter
            self.block = 0
            
            # Monitoring syscall
            if self.askme :
                # Engine check itself dangerous arguments
                if not self.askme_para :
                    self.emul.add_syscall(process, syscall, 2)
                    self.emul.emulate_syscall_enter(process, syscall)
                    self.block = self.emul.get_block(process, syscall)              
                # Asked the user
                else :
                    self.block = self.syscall_askme(process, syscall)
                    self.emul.add_syscall(process, syscall, self.block)
                    self.emul.emulate_syscall_enter(process, syscall)
            else :
                # Add syscall and emulate it
                self.emul.add_syscall(process, syscall, self.block)
                self.emul.emulate_syscall_enter(process, syscall)

            self.nb += 1
            debug("%s enters %s" % (process, syscall))
            self.active_syscalls[process] = syscall

        # Break at next syscall
        process.syscall()


    def processExited(self, event):
        # Display syscall which has not exited
        state = event.process.syscall_state
        process = event.process
        self.displaySyscall(process, state.syscall, False)

        # Display exit message
        warning("*** %s ***" % event)
