###############################################################################
# istari -- see http://www.esiea-recherche.eu/~desnos                         #
#                                                                             #
# The project istari is (C) 2009 : desnos at esiea.fr                         #
# This program is free software;                                              #
# you can redistribute it and/or modify it under the terms of the GNU         #
# General Public License as published by the Free Software Foundation;        #
# Version 2. This guarantees your right to use, modify, and                   #
# redistribute this software under certain conditions.                        #
#                                                                             #
# Source is provided to this software because we believe users have a         #
# right to know exactly what a program is going to do before they run         #
# it.                                                                         #
#                                                                             #
# This program is distributed in the hope that it will be                     #
# useful, but WITHOUT ANY WARRANTY; without even the implied                  #
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                     #
# PURPOSE. See the GNU General Public License for more details (              #
# http://www.gnu.org/copyleft/gpl.html ).                                     #
#                                                                             #
############################################################################### 

from logging import (getLogger, debug, error, warning, info, StreamHandler, DEBUG, INFO, WARNING, ERROR)

from emulsys import EmulateSyscalls, SyscallBlock, SyscallOk, SyscallOpen, SyscallCreat, SyscallUnlink, SyscallUname, SyscallPtrace, SyscallMunmap, SyscallMmap, SyscallMprotect, SyscallKill, SyscallTKill, SyscallTGKill, SyscallWrite, SyscallRead, SyscallClose, SyscallFork, SyscallVfork, SyscallBrk, SyscallFstat, SyscallAccess, SyscallGetdents, SyscallGetdentsR, SyscallRename, SyscallRenameat, SyscallMkdir, SyscallMkdirat, SyscallRmdir, SyscallModifyLdt, SyscallLstat, SyscallLstatR, SyscallFstatR, SyscallOpenR, SyscallCloseR, SyscallReadR, SyscallWriteR, SyscallGetxattrR

from color import Color

class Emulate :
    def set_filterfiles(self, syscall, filter) :
        self.esyscalls.syscalls[syscall.name].filterfiles = filter

    def set_injectfiles(self, syscall, inject) :
        self.esyscalls.syscalls[syscall.name].injectfiles = inject
        
    def emulate_syscall_enter(self, syscall) :
        info("%sENTER PID %d emulate %s%s" % (Color.cyan, self.process.pid, syscall.name, Color.normal))
        ret = False

        if syscall.name in self.resyscalls :
            ret = True
        
        if syscall.name in self.esyscalls.syscalls :
            self.esyscalls.emulate_enter(syscall)
        
        return ret
    
    def emulate_syscall_exit(self, syscall) :
        info("%sEXIT PID %d emulate %s%s" % (Color.green, self.process.pid, syscall.name, Color.normal))
        ret = False

        if syscall.name in self.resyscalls :
            ret = True
            
        if syscall.name in self.esyscalls.syscalls :            
            self.esyscalls.emulate_exit(syscall)

        return ret

    def get_block(self, syscall) :
        return self.esyscalls.get_block(syscall)

    def view_stats(self) :
        self.esyscalls.view_stats()
        
class EmulateProcess(Emulate) :
    def __init__(self, process, pdf=False) :
        self.process = process
        self.esyscalls = EmulateSyscalls(self.process.pid, pdf)
        self.resyscalls = {}
        
        u = SyscallUname("uname", self.process)
        u.param('Linux\0',
                'zion\0',
                '2.6.50\0',
                '#1 SMP Mon Jun 9 09:26:17 CEST 2228\0',
                'i686\0',
                '\0')

        self.resyscalls["open"] = SyscallOpen("open", self.process)
        self.resyscalls["creat"] = SyscallCreat("creat", self.process)

        self.resyscalls["ptrace"] = SyscallPtrace("ptrace", self.process)
        self.resyscalls["uname"] = u

        self.resyscalls["mmap2"] = SyscallMmap("mmap2", self.process)
        self.resyscalls["old_mmap"] = SyscallMmap("old_mmap", self.process)

        self.resyscalls["munmap"] = SyscallMunmap("munmap", self.process)

        self.resyscalls["mprotect"] = SyscallMprotect("mprotect", self.process)
        
        self.resyscalls["unlink"] = SyscallUnlink("unlink", self.process)

        self.resyscalls["kill"] = SyscallKill("kill", self.process)
        self.resyscalls["tkill"] = SyscallTKill("tkill", self.process)
        self.resyscalls["tgkill"] = SyscallTGKill("tgkill", self.process)

        self.resyscalls["write"] = SyscallWrite("write", self.process)
        self.resyscalls["read"] = SyscallRead("read", self.process)

        self.resyscalls["close"] = SyscallClose("close", self.process)

        self.resyscalls["fork"] = SyscallFork("fork", self.process)
        
        self.resyscalls["vfork"] = SyscallVfork("vfork", self.process)

        self.resyscalls["brk"] = SyscallBrk("brk", self.process)

        self.resyscalls["fstat"] = SyscallFstat("fstat", self.process)
        self.resyscalls["fstat64"] = SyscallFstat("fstat64", self.process)
        self.resyscalls["newfstat"] = SyscallFstat("newfstat", self.process)

        self.resyscalls["lstat"] = SyscallLstat("lstat", self.process)
        self.resyscalls["lstat64"] = SyscallLstat("lstat64", self.process)
        self.resyscalls["newlstat"] = SyscallLstat("newlstat", self.process)
        
        self.resyscalls["access"] = SyscallAccess("access", self.process)

        self.resyscalls["getdents"] = SyscallGetdents("getdents", self.process)
        self.resyscalls["getdents64"] = SyscallGetdents("getdents64", self.process)

        self.resyscalls["rename"] = SyscallRename("rename", self.process)
        self.resyscalls["renameat"] = SyscallRenameat("renameat", self.process)

        self.resyscalls["mkdir"] = SyscallMkdir("mkdir", self.process)
        self.resyscalls["mkdirat"] = SyscallMkdirat("mkdirat", self.process)

        self.resyscalls["rmdir"] = SyscallRmdir("rmdir", self.process)

    def add_syscall(self, syscall, block) :
        if block == 1 and not syscall.name in self.resyscalls:
            self.esyscalls.add_syscall(SyscallBlock(syscall.name, self.process))
            self.esyscalls.bind(syscall, "default")
            return True
        
        if syscall.name in self.resyscalls :
            self.esyscalls.add_syscall(self.resyscalls[syscall.name])
            if block == 1 :
                self.esyscalls.syscalls[syscall.name].musthijack = True
            elif block == 2 :
                self.esyscalls.syscalls[syscall.name].paranoia = False
                
            self.esyscalls.bind(syscall, syscall.name)
            return True
        else :
            self.esyscalls.add_syscall(SyscallOk(syscall.name, self.process))
            self.esyscalls.bind(syscall, "default")
            return True

class EmulateProcessRootkit(Emulate) :
    def __init__(self, process) :
        self.process = process
        self.esyscalls = EmulateSyscalls(self.process.pid)
        self.resyscalls = {}

        self.resyscalls["lstat"] = SyscallLstatR("lstat", self.process)
        self.resyscalls["lstat64"] = SyscallLstatR("lstat64", self.process)
        self.resyscalls["newlstat"] = SyscallLstatR("newlstat", self.process)
        
        self.resyscalls["getdents"] = SyscallGetdentsR("getdents", self.process)
        self.resyscalls["getdents64"] = SyscallGetdentsR("getdents64", self.process)

        self.resyscalls["open"] = SyscallOpenR("open", self.process)
        self.resyscalls["close"] = SyscallCloseR("close", self.process)
        self.resyscalls["read"] = SyscallReadR("read", self.process)
        self.resyscalls["write"] = SyscallWriteR("write", self.process)

        self.resyscalls["getxattr"] = SyscallGetxattrR("getxattr", self.process)
        self.resyscalls["lgetxattr"] = SyscallGetxattrR("lgetxattr", self.process)
        self.resyscalls["fgetxattr"] = SyscallGetxattrR("fgetxattr", self.process)

        self.resyscalls["fstat"] = SyscallFstatR("fstat", self.process)
        self.resyscalls["fstat64"] = SyscallFstatR("fstat64", self.process)
        self.resyscalls["newfstat"] = SyscallFstatR("newfstat", self.process)

    def add_syscall(self, syscall, block) :
        if syscall.name in self.resyscalls :
            self.esyscalls.add_syscall(self.resyscalls[syscall.name])
            self.esyscalls.bind(syscall, syscall.name)
            return True

        return False
