#!/usr/bin/env python

###############################################################################
# pallando -- see http://www.esiea-recherche.eu/~desnos                       #
#                                                                             #
# The project pallando is (C) 2008 : desnos at esiea.fr                       #
# This program is free software;                                              #
# you can redistribute it and/or modify it under the terms of the GNU         #
# General Public License as published by the Free Software Foundation;        #
# Version 2. This guarantees your right to use, modify, and                   #
# redistribute this software under certain conditions.                        #
#                                                                             #
# Source is provided to this software because we believe users have a         #
# right to know exactly what a program is going to do before they run         #
# it.                                                                         #
#                                                                             #
# This program is distributed in the hope that it will be                     #
# useful, but WITHOUT ANY WARRANTY; without even the implied                  #
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                     #
# PURPOSE. See the GNU General Public License for more details (              #
# http://www.gnu.org/copyleft/gpl.html ).                                     #
#                                                                             #
# Dependency :                                                                #
#                                                                             #
###############################################################################

import sys, socket

from logging import (log, info, error, NOTSET, CRITICAL, DEBUG, INFO, WARNING, ERROR)
from log import NOLOG, MyLog
from color import Color

from linux_struct import stat64, dirent
from ctypes import sizeof, cast, POINTER, c_char_p
from emulsys import GFile

class SPFile :
    def __init__(self, name, flags, obj) :
        self.name = name
        self.flags = flags
        self.obj = obj

class Directory :
    def __init__(self) :
        self.l = []

    def view(self) :
        for i in self.l :
            i.view()

class SyscallProxyClient :
    def __init__(self, host, port) :
        self.host = host
        self.port = port
        self.open = {}

    def sp_open(self, name, flags) :
        self.s.send("Open::" + name + "::" + flags)
        data = self.s.recv(1024)
        return int(data)

    def sp_close(self, fd) :
        self.s.send("Close::" + "%d" % fd)
        data = self.s.recv(1024)
        return int(data)

    def sp_read(self, fd, size) :
        self.s.send("Read::" + "%d" % fd + "::" + "%d" % size)
        ret = self.s.recv(32)
        buf = ""
        if not ret.isdigit() :
            i = 0
            while i < len(ret) :
                if not ret[i].isdigit() :
                    break
                i += 1
            buf += ret[i:]
            ret = int(ret[0:i])
        else :
            ret = int(ret)

        buf += self.s.recv(size)
        n = ""
        lenght = len(buf)
        while lenght < ret :
            n += self.s.recv(size - len(buf))
            lenght += len(n)
            buf += n

        return [ret, buf]

    def sp_getuid(self) :
        self.s.send("Getuid::")
        ret = self.s.recv(1024)
        return int(ret)

    def sp_fstat(self, fd) :
        self.s.send("Fstat::" + "%d" % fd)
        ret = self.s.recv(1024)
        mystat = cast(ret, POINTER(stat64))[0]
        return mystat

    def sp_getcwd(self) :
        self.s.send("Getcwd::255")
        ret = self.s.recv(32)
        buf = self.s.recv(255)
        return [ret, buf]

    def sp_exit(self) :
        self.s.send("Exit::" + "0")
        ret = self.s.recv(32)
        return ret

    def sp_chdir(self, path) :
        self.s.send("Chdir::" + path)
        ret = self.s.recv(32)
        return ret

    def sp_unlink(self, path) :
        self.s.send("Unlink::" + path)
        ret = self.s.recv(32)
        return ret

    def sp_getdents(self, fd) :
        self.s.send("Getdents::" + "%d" % fd)
        ret = self.s.recv(32)
        buf = ""
        if not ret.isdigit() :
            i = 0
            while i < len(ret) :
                if not ret[i].isdigit() :
                    break
                i += 1
            buf += ret[i:]
            ret = int(ret[0:i])
        else :
            ret = int(ret)

#        print ret
        buf += self.s.recv(ret)
#        print repr(buf)
        #print ret
        return [ "%d" % ret, buf ]

    def cmd_cat(self, fichier) :
        fd = self.sp_open(fichier, 'r')
        if fd >= 0 :
            mystat = self.sp_fstat(fd)
        #print "Size %d" % mystat.st_size
            ret = self.sp_read(fd, mystat.st_size)
            print ret[1]
            self.sp_close(fd)

    def cmd_pwd(self) :
        ret = self.sp_getcwd()
        #print ret[1]
        return ret[1]

    def cmd_cd(self, path) :
        ret = self.sp_chdir(path)

    def cmd_rm(self, path) :
        ret = self.sp_unlink(path)

    def cmd_exit(self) :
        ret = self.sp_exit()

    def cmd_ls(self, path) :
        d = Directory()
        fd = self.sp_open(path, 'r')
        if fd >= 0 :
            ret = self.sp_getdents(fd)
            size = int(ret[0])
            buf = ret[1]

            nb = 0
            l = 0
            while l < size :
                data = cast(buf[l:l+sizeof(dirent)], POINTER(dirent))[0]
                mydirent = dirent()
                mydirent.d_ino = data.d_ino
                mydirent.d_off = data.d_off
                mydirent.d_reclen = data.d_reclen
                mydirent.d_type = data.d_type
                mydirent.d_name = data.d_name

                rlen = data.d_reclen

                name = buf[l+dirent.d_name.offset:l+rlen]

                try :
                    name = name[:name.index('\0')]
                    d.l.append(GFile(mydirent, name))
                except ValueError :
                    print "ERROR"
                    
                l += rlen
                nb += 1

#            print nb
            d.view()

            self.sp_close(fd)


    def shell(self) :
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.connect((self.host, self.port))
        self.s.send('New::SyscallProxy')
        data = self.s.recv(1024)
        print data
        prompt = "%d" % self.sp_getuid() + ":" + self.cmd_pwd() + " >> "
        while 1 :
            print prompt,
            l = sys.stdin.readline().split()
            if l[0] == "pwd" :
                print self.cmd_pwd()
            elif l[0] == "ls" :
                if len(l) > 1 :
                    self.cmd_ls(l[1])
                else :
                    self.cmd_ls(".")
            elif l[0] == "cd" :
                if len(l) > 1 :
                    self.cmd_cd(l[1])
                    prompt = "%d" % self.sp_getuid() + ":" + self.cmd_pwd() + " >> "
            elif l[0] == "rm" :
                if len(l) > 1 :
                    self.cmd_rm(l[1])
            elif l[0] == "cat" :
                if len(l) > 1 :
                    self.cmd_cat(l[1])
            elif l[0] == "exit" :
                self.cmd_exit()
                break
            elif l[0] == "quit" :
                break
            


    def run(self, cmd) :
        self.s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.s.connect((self.host, self.port))
        self.s.send('New::SyscallProxy')
        data = self.s.recv(1024)

        if cmd[0] == "pwd" :
            print self.cmd_pwd()
        elif cmd[0] == "ls" :
            if len(cmd) > 1 :
                self.cmd_ls(cmd[1])
            else :
                self.cmd_ls(".")
        elif cmd[0] == "cd" :
            if len(cmd) > 1 :
                self.cmd_cd(cmd[1])
        elif cmd[0] == "rm" :
            if len(cmd) > 1 :
                self.cmd_rm(cmd[1])
        elif cmd[0] == "cat" :
            if len(cmd) > 1 :
                self.cmd_cat(cmd[1])
        elif cmd[0] == "exit" :
            self.cmd_exit()
        else :
            info("Unknown command " + cmd[0])
        self.s.close()

if __name__ == "__main__" :
    mylog = MyLog()
    mylog.setLevel(INFO)

    if len(sys.argv) >= 3 :
        spc = SyscallProxyClient(sys.argv[2], int(sys.argv[3]))
        
        if len(sys.argv) >= 4 :
            if sys.argv[1] == "-s" :
                spc.shell()

            elif len(sys.argv) > 4 :
                if sys.argv[1] == "-r" :
                    spc.run(sys.argv[4:])
