###############################################################################
# istari -- see http://www.esiea-recherche.eu/~desnos                         #
#                                                                             #
# The project istari is (C) 2009 : desnos at esiea.fr                         #
# This program is free software;                                              #
# you can redistribute it and/or modify it under the terms of the GNU         #
# General Public License as published by the Free Software Foundation;        #
# Version 2. This guarantees your right to use, modify, and                   #
# redistribute this software under certain conditions.                        #
#                                                                             #
# Source is provided to this software because we believe users have a         #
# right to know exactly what a program is going to do before they run         #
# it.                                                                         #
#                                                                             #
# This program is distributed in the hope that it will be                     #
# useful, but WITHOUT ANY WARRANTY; without even the implied                  #
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR                     #
# PURPOSE. See the GNU General Public License for more details (              #
# http://www.gnu.org/copyleft/gpl.html ).                                     #
#                                                                             #
############################################################################### 

from logging import (getLogger, debug, error, warning, info, StreamHandler, DEBUG, INFO, WARNING, ERROR)

from color import Color
import os, popen2

class OArg :
    def __init__(self) :
        self.l = []

    def add(self, arg) :
        self.l.append(arg)
        
class Access :
    def __init__(self, name) :
        self.oarg = []
        self.name = name
        
    def view_access(self, access) :
        if access == True :
            return(Color.green + " ===> " + "Authorized")
        else :
            return(Color.red + " ===> " + "Refused")

    def format_access(self, access) :
        if access == True :
            return(" ===> " + "Authorized")
        else :
            return(" ===> " + "Refused")

    def view(self):
        if self.oarg == [] :
            return

        self.view_banner("Stats %s :" % self.name)
        self.view_arg()

    def pdf(self) :
        if self.oarg == [] :
            return None

        format = self.format_banner("Stats %s:" % self.name)
        format += "\\begin{verbatim}\n"

        format += self.format_arg()
        format += "\\end{verbatim}\n"
        
        return format
    
    def view_banner(self, banner) :
        info("%s\n\t%s%s" % (Color.purple, banner, Color.normal))

    def view_arg(self) :
        for i in self.oarg :
            end = i.l.pop(len(i.l) - 1)
            arg = ""
            for j in i.l :
                arg += j
                arg += " "
            arg += self.view_access(end)
            i.l.append(end)
            info("%s%s%s" % (Color.blue, arg, Color.normal))

    def format_banner(self, banner) :
        return ("\\chapter{%s}\n" % banner)
    
    def format_arg(self) :
        format = ""
        for i in self.oarg :
            end = i.l.pop(len(i.l) - 1)
            arg = ""
            for j in i.l :
                arg += j
                arg += " "
            arg += self.format_access(end)
            i.l.append(end)
            format += arg
            format += "\n"
        return format
    
class StatsOpen(Access) :
    def add(self, path, mode, access) :
        o = OArg()
        o.add(path)
        o.add(mode)
        o.add(access)
        
        self.oarg.append(o)

class StatsCreat(Access) :
    def add(self, path, mode, access) :
        o = OArg()
        o.add(path)
        o.add(mode)
        o.add(access)
        
        self.oarg.append(o)
    
class StatsPtrace(Access) :
    def add(self, request, pid, addr, data, access) :
        o = OArg()
        o.add(request)
        o.add(pid)
        o.add(addr)
        o.add(data)
        o.add(access)

        self.oarg.append(o)

class StatsMprotect(Access) :
    def add(self, start, length, prot, access) :
        o = OArg()
        o.add(start)
        o.add(length)
        o.add(prot)
        o.add(access)

        self.oarg.append(o)
        
class StatsMunmap(Access) :
    def add(self, start, length, access) :
        o = OArg()
        o.add(start)
        o.add(length)
        o.add(access)

        self.oarg.append(o)

class StatsMmap(Access) :
    def add(self, start, length, prot, flags, fd, offset, access) :
        o = OArg()
        o.add(start)
        o.add(length)
        o.add(prot)
        o.add(flags)
        o.add(fd)
        o.add(offset)
        o.add(access)

        self.oarg.append(o)

class StatsUnlink(Access) :
    def add(self, path, access) :
        o = OArg()
        o.add(path)
        o.add(access)

        self.oarg.append(o)

class StatsKill(Access) :
    def add(self, pid, sig, access) :
        o = OArg()
        o.add(pid)
        o.add(sig)
        o.add(access)

        self.oarg.append(o)

class StatsTKill(Access) :
    def add(self, tid, sig, access) :
        o = OArg()
        o.add(tid)
        o.add(sig)
        o.add(access)

        self.oarg.append(o)

class StatsTGKill(Access) :
    def add(self, tgid, tid, sig, access) :
        o = OArg()
        o.add(tgid)
        o.add(tid)
        o.add(sig)
        o.add(access)

        self.oarg.append(o)
        
class StatsWrite(Access) :
    def add(self, fd, content, length, access) :
        o = OArg()
        o.add(fd)
        o.add(content)
        o.add(length)
        o.add(access)

        self.oarg.append(o)

class StatsRead(Access) :
    def add(self, fd, content, length, access) :
        o = OArg()
        o.add(fd)
        o.add(content)
        o.add(length)
        o.add(access)

        self.oarg.append(o)
        
class StatsClose(Access) :
    def add(self, fd, access) :
        o = OArg()
        o.add(fd)
        o.add(access)

        self.oarg.append(o)

class StatsUname(Access) :
    def add(self, new_utsname, access) :
        o = OArg()
        o.add(new_utsname)
        o.add(access)

        self.oarg.append(o)

class StatsFork(Access) :
    def add(self, access) :
        o = OArg()
        o.add(access)

        self.oarg.append(o)

class StatsVfork(Access) :
    def add(self, access) :
        o = OArg()
        o.add(access)

        self.oarg.append(o)
        
class StatsBrk(Access) :
    def add(self, end_data_segement, access) :
        o = OArg()
        o.add(end_data_segement)
        o.add(access)

        self.oarg.append(o)

class StatsFstat(Access) :
    def add(self, fd, statbuf, access) :
        o = OArg()
        o.add(fd)
        o.add(statbuf)
        o.add(access)

        self.oarg.append(o)

class StatsLstat(Access) :
    def add(self, path, statbuf, access) :
        o = OArg()
        o.add(path)
        o.add(statbuf)
        o.add(access)

        self.oarg.append(o)
        
class StatsAccess(Access) :
    def add(self, pathname, mode, access) :
        o = OArg()
        o.add(pathname)
        o.add(mode)
        o.add(access)

        self.oarg.append(o)

class StatsGetdents(Access) :
    def add(self, fd, dirp, count, access) :
        o = OArg()
        o.add(fd)
        o.add(dirp)
        o.add(count)
        o.add(access)

        self.oarg.append(o)

class StatsRename(Access) :
    def add(self, oldpath, newpath, access) :
        o = OArg()
        o.add(oldpath)
        o.add(newpath)
        o.add(access)

        self.oarg.append(o)

class StatsRenameat(Access) :
    def add(self, olddirfd, oldpath, newdirfd, newpath, access) :
        o = OArg()
        o.add(olddirfd)
        o.add(oldpath)
        o.add(newdirfd)
        o.add(newpath)
        o.add(access)

        self.oarg.append(o)

class StatsMkdir(Access) :
    def add(self, pathname, mode, access) :
        o = OArg()
        o.add(pathname)
        o.add(mode)
        o.add(access)

        self.oarg.append(o)

class StatsMkdirat(Access) :
    def add(self, dirfd, pathname, mode, access) :
        o = OArg()
        o.add(dirfd)
        o.add(pathname)
        o.add(mode)
        o.add(access)

        self.oarg.append(o)

class StatsRmdir(Access) :
    def add(self, pathname, access) :
        o = OArg()
        o.add(pathname)
        o.add(access)

        self.oarg.append(o)

class StatsModifyLdt(Access) :
    def add(self, func, ptr, bytescount, access) :
        o = OArg()
        o.add(func)
        o.add(ptr)
        o.add(bytescount)
        o.add(access)

        self.oarg.append(o)
        
class StatsDefault(Access) :
    def __init__(self) :
        self.odef = {}

    def add(self, name, content, access) :
        if name not in self.odef :
            self.odef[name] = []

        o = OArg()
        o.add(content)
        o.add(access)
        
        self.odef[name].append(o)

    def view(self):
        if self.odef == {} :
            return

        self.view_banner("Stats Default :")
        for i in self.odef :
            self.oarg = self.odef[i]
            self.view_arg()

class Sys :
    def __init__(self, name, stats) :
        self.l = []
        self.l.append(name)
        self.stats = stats
        
    def link(self, name) :
        self.l.append(name)

    def view(self) :
        self.stats.view()

    def pdf(self) :
        return self.stats.pdf()
        
class StatsSyscall :
    def __init__(self, pid, pdf=False) :
        self.pid = pid
        self.pdf = pdf
        self.syscalls = {}
        self.syscalls["open"] = Sys("open", StatsOpen("open"))
        self.syscalls["creat"] = Sys("creat", StatsCreat("creat"))
        self.syscalls["ptrace"] = Sys("ptrace", StatsPtrace("ptrace"))
        
        self.syscalls["mmap2"] = Sys("mmap2", StatsMmap("mmap"))
        self.syscalls["mmap2"].link("old_mmap")

        self.syscalls["munmap"] = Sys("munmap", StatsMunmap("munmap"))

        self.syscalls["mprotect"] = Sys("mprotect", StatsMprotect("mprotect"))
        
        self.syscalls["unlink"] = Sys("unlink", StatsUnlink("unlink"))

        self.syscalls["kill"] = Sys("kill", StatsKill("kill"))
        self.syscalls["tkill"] = Sys("tkill", StatsTKill("tkill"))
        self.syscalls["tgkill"] = Sys("tgkill", StatsTGKill("tgkill"))

        self.syscalls["write"] = Sys("write", StatsWrite("write"))
        self.syscalls["read"] = Sys("read", StatsRead("read"))

        self.syscalls["close"] = Sys("close", StatsClose("close"))

        self.syscalls["uname"] = Sys("uname", StatsUname("uname"))
        self.syscalls["uname"].link("olduname")
        self.syscalls["uname"].link("oldolduname")

        self.syscalls["fork"] = Sys("fork", StatsFork("fork"))

        self.syscalls["vfork"] = Sys("vfork", StatsVfork("vfork"))

        self.syscalls["brk"] = Sys("brk", StatsBrk("brk"))

        self.syscalls["fstat"] = Sys("fstat", StatsFstat("fstat"))
        self.syscalls["fstat"].link("fstat64")
        self.syscalls["fstat"].link("newfstat")

        self.syscalls["lstat"] = Sys("lstat", StatsLstat("lstat"))
        self.syscalls["lstat"].link("lstat64")
        self.syscalls["lstat"].link("newlstat")
          
        self.syscalls["access"] = Sys("access", StatsAccess("access"))

        self.syscalls["getdents"] = Sys("getdents", StatsGetdents("getdents"))
        self.syscalls["getdents"].link("getdents64")

        self.syscalls["rename"] = Sys("rename", StatsRename("rename"))
        self.syscalls["renameat"] = Sys("renameat", StatsRenameat("renameat"))

        self.syscalls["mkdir"] = Sys("mkdir", StatsMkdir("mkdir"))
        self.syscalls["mkdirat"] = Sys("mkdirat", StatsMkdirat("mkdirat"))

        self.syscalls["rmdir"] = Sys("rmdir", StatsRmdir("rmdir"))

        self.syscalls["modify_ldt"] = Sys("modify_ldt", StatsModifyLdt("modify_ldt"))

        self.syscalls["default"] = Sys("default", StatsDefault())
        
        
    def get_stats(self, name) :
        for i in self.syscalls :
            if name == i :
                return self.syscalls[name].stats

            for j in self.syscalls[i].l :
                if name == j :
                    return self.syscalls[i].stats

        return self.syscalls["default"].stats

    def view(self) :
        self.syscalls["default"].view()
        for i in self.syscalls :
            if i != "default" :
                self.syscalls[i].view()

        if self.pdf :
            pdf = StatsFormatPdf(self.pid, "Olorin The Istari", ("Syscalls pid %d" % self.pid), self)
            pdf.make()
        
class StatsFormatPdf :
    def __init__(self, pid, auteur, titre, stats) :
        self.pid = pid
        self.auteur = auteur
        self.titre = titre
        self.stats = stats

        self.beginlatex = "\documentclass[a4paper]{report}\n"   \
                          "\usepackage{a4wide}\n"               \
                          "\usepackage[a4paper]{geometry}\n"    \
                          "\usepackage{amssymb}\n"              \
                          "\usepackage[francais]{babel}\n"      \
                          "\usepackage[utf8]{inputenc}\n"       \
                          "\usepackage{makeidx}\n"              \
                          "\usepackage{graphicx}\n"             \
                          "\usepackage{enumerate}\n"            \
                          "\usepackage{listings}\n"             

        self.auteurlatex = ("\\author{%s}\n" % self.auteur)

        self.titrelatex = ("\\title{%s}\n" % self.titre)

        self.middlelatex = "\\begin{document}\n"                 \
                           "\\pagestyle{headings}\n"              \
                           "\\maketitle\n"                        \
                           "\\tableofcontents\n"                  \
                           "\\newpage\n"                          \


        self.endlatex = "\end{document}\n"

    def make(self) :
        print "Making pdf2latex..."
        self.latex = self.beginlatex + self.auteurlatex + self.titrelatex + self.middlelatex

        for i in self.stats.syscalls :
            if i != "default" :
                ret = self.stats.syscalls[i].pdf()
                if ret != None :
                    self.latex += ret
                    self.latex += "\n"
                    

        self.latex += self.endlatex

        fd = open("./syscalls-%d.tex" % self.pid, "w")
        fd.write(self.latex)
        fd.close()

        nbpass = 0
        while nbpass < 2 : 
            i, o = os.popen2("/usr/bin/pdflatex" + " ./syscalls-%d.tex" % self.pid)
            for j in o :
                pass
            
            o.close()
            i.close()

            nbpass += 1
